from __future__ import annotations

from datetime import datetime, timezone
from typing import TYPE_CHECKING, Any
from uuid import uuid4

from checkov.common.output.cyclonedx_consts import SCA_CHECKTYPES
from checkov.common.util.http_utils import valid_url
from checkov.version import version

if TYPE_CHECKING:
    from checkov.common.output.record import Record
    from checkov.common.output.report import Report

DEFAULT_SEVERITY_GITLAB_LEVEL = "Unknown"
SEVERITY_TO_GITLAB_LEVEL = {
    "critical": "Critical",
    "high": "High",
    "medium": "Medium",
    "low": "Low",
    "none": "Info",
}


class GitLabSast:
    def __init__(self, reports: list[Report]):
        self.reports = reports

        self.sast_json = self.create_sast_json()

    def create_sast_json(self) -> dict[str, Any]:
        return {
            "schema": "https://gitlab.com/gitlab-org/security-products/security-report-schemas/-/raw/v15.0.4/dist/sast-report-format.json",
            "version": "15.0.4",
            "scan": self._create_scan(),
            "vulnerabilities": self._create_vulnerabilities(),
        }

    def _create_scan(self) -> dict[str, Any]:
        current_datetime = datetime.now(timezone.utc).strftime("%Y-%m-%dT%H:%M:%S")
        scanner = {
            "id": "checkov",
            "name": "Checkov",
            "url": "https://www.checkov.io/",
            "vendor": {
                "name": "Bridgecrew",
            },
            "version": version,
        }

        return {
            "start_time": current_datetime,  # needs to be done properly in a later stage
            "end_time": current_datetime,
            "analyzer": scanner,  # same for us
            "scanner": scanner,
            "status": "success",
            "type": "sast",
        }

    def _create_vulnerabilities(self) -> list[dict[str, Any]]:
        vulnerabilities = []

        for report in self.reports:
            if report.check_type in SCA_CHECKTYPES:
                for check in report.failed_checks:
                    vulnerability = None
                    if check.check_id.startswith("BC_LIC"):
                        vulnerability = self._create_license_vulnerability(record=check)
                    elif check.check_id.startswith(("BC_VUL", "CKV_CVE")):
                        vulnerability = self._create_cve_vulnerability(record=check)

                    if vulnerability:
                        vulnerabilities.append(vulnerability)
            else:
                for check in report.failed_checks:
                    vulnerabilities.append(self._create_iac_vulnerability(record=check))

        return vulnerabilities

    def _create_iac_vulnerability(self, record: Record) -> dict[str, Any]:
        severity = record.severity.name.lower() if record.severity else ""

        vulnerability: "dict[str, Any]" = {
            "id": str(uuid4()),
            "identifiers": [
                {
                    "name": record.check_id,
                    "type": "checkov",
                    "value": record.check_id,
                }
            ],
            "location": {
                "file": record.repo_file_path.lstrip("/"),
                "start_line": record.file_line_range[0],
                "end_line": record.file_line_range[1],
            },
            "name": record.check_name,
            "description": f"Further info can be found {record.guideline}",
            "severity": SEVERITY_TO_GITLAB_LEVEL.get(severity, DEFAULT_SEVERITY_GITLAB_LEVEL),
            "solution": f"Further info can be found {record.guideline}",
        }

        link = record.guideline
        if valid_url(link):
            # url can't be None
            vulnerability["identifiers"][0]["url"] = link
            vulnerability["links"] = [
                {
                    "url": link,
                }
            ]

        return vulnerability

    def _create_cve_vulnerability(self, record: Record) -> dict[str, Any] | None:
        details = record.vulnerability_details
        if not details:
            # this shouldn't happen
            return None

        severity = record.severity.name.lower() if record.severity else ""

        vulnerability: "dict[str, Any]" = {
            "id": str(uuid4()),
            "identifiers": [
                {
                    "name": record.short_description,
                    "type": "cve",
                    "value": details["id"],
                }
            ],
            "location": {
                "file": record.repo_file_path.lstrip("/"),
            },
            "name": record.short_description,
            "description": details.get("description"),
            "severity": SEVERITY_TO_GITLAB_LEVEL.get(severity, DEFAULT_SEVERITY_GITLAB_LEVEL),
            "solution": details.get("status"),
        }

        link = details.get("link")
        if valid_url(link):
            # url can't be None
            vulnerability["identifiers"][0]["url"] = link
            vulnerability["links"] = [
                {
                    "url": link,
                }
            ]

        return vulnerability

    def _create_license_vulnerability(self, record: Record) -> dict[str, Any] | None:
        details = record.vulnerability_details
        if not details:
            # this shouldn't happen
            return None

        return {
            "id": str(uuid4()),
            "identifiers": [
                {
                    "name": record.check_id,
                    "type": "license",
                    "value": record.check_id,
                }
            ],
            "location": {
                "file": record.repo_file_path.lstrip("/"),
            },
            "name": record.short_description,
            "description": f"Package {details['package_name']}@{details['package_version']} has license {details['license']}",
        }



from __future__ import annotations

from typing import Iterable, Any
from abc import abstractmethod
from checkov.openapi.checks.base_openapi_check import BaseOpenapiCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class BaseOpenapiCheckV2(BaseOpenapiCheck):
    def __init__(self, name: str, id: str, categories: Iterable[CheckCategories], supported_entities: Iterable[str],
                 block_type: str) -> None:
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_entities=supported_entities,
            block_type=block_type,
        )

    @abstractmethod
    def scan_openapi_conf(self, conf: dict[str, Any], entity_type: str) -> tuple[CheckResult, dict[str, Any]]:
        raise NotImplementedError()

    def scan_entity_conf(self, conf: dict[str, Any], entity_type: str) -> tuple[CheckResult, dict[str, Any]]:
        if "swagger" in conf:
            swagger = conf.get("swagger")
            if isinstance(swagger, str) and swagger == '2.0':
                return self.scan_openapi_conf(conf, entity_type)
        return CheckResult.UNKNOWN, conf




from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class WildcardRoles(BaseResourceCheck):

    def __init__(self):
        name = "Minimize wildcard use in Roles and ClusterRoles"
        id = "CKV_K8S_49"
        supported_resources = ["kubernetes_role", "kubernetes_role_v1",
                               "kubernetes_cluster_role", "kubernetes_cluster_role_v1"]

        categories = [CheckCategories.IAM]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf) -> CheckResult:
        rules = conf.get("rule")
        if isinstance(rules, list) and rules:
            for rule in rules:
                if "api_groups" in rule:
                    if rule["api_groups"] is not None and "*" in rule["api_groups"][0]:
                        return CheckResult.FAILED
                if "resources" in rule:
                    if rule["resources"] is not None and "*" in rule["resources"][0]:
                        return CheckResult.FAILED
                if "verbs" in rule:
                    if rule["verbs"] is not None and "*" in rule["verbs"][0]:
                        return CheckResult.FAILED

        return CheckResult.PASSED


check = WildcardRoles()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.kubernetes.checks.resource.base_spec_check import BaseK8Check
from checkov.common.util.type_forcers import force_list


class NginxIngressCVE202125742Alias(BaseK8Check):
    def __init__(self) -> None:
        name = "Prevent NGINX Ingress annotation snippets which contain alias statements See CVE-2021-25742"
        id = "CKV_K8S_154"
        supported_kind = ("Ingress",)
        categories = (CheckCategories.KUBERNETES,)
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_kind)

    def scan_spec_conf(self, conf: dict[str, Any]) -> CheckResult:
        metadata = conf.get("metadata")
        if metadata:
            annotations = metadata.get("annotations")
            if annotations:
                for annotation in force_list(annotations):
                    if any("snippet" in key and "alias" in value for key, value in annotation.items()):
                        return CheckResult.FAILED
        return CheckResult.PASSED


check = NginxIngressCVE202125742Alias()



from typing import List, Any

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class CloudFrontTLS12(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Verify CloudFront Distribution Viewer Certificate is using TLS v1.2"
        id = "CKV_AWS_174"
        supported_resources = ("aws_cloudfront_distribution",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "viewer_certificate/[0]/minimum_protocol_version"

    def get_expected_values(self) -> List[Any]:
        return ["TLSv1.2_2018", "TLSv1.2_2019", "TLSv1.2_2021"]

    def get_expected_value(self) -> Any:
        return "TLSv1.2_2021"


check = CloudFrontTLS12()



from checkov.common.models.consts import ANY_VALUE
from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from typing import Any


class AzureFirewallDefinesPolicy(BaseResourceValueCheck) :
    def __init__(self) -> None:
        name = "Ensure Firewall defines a firewall policy"
        id = "CKV_AZURE_219"
        supported_resources = ['azurerm_firewall']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "firewall_policy_id"

    def get_expected_value(self) -> Any:
        return ANY_VALUE


check = AzureFirewallDefinesPolicy()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AppServiceAlwaysOn(BaseResourceValueCheck):
    def __init__(self) -> None:
        # "Azure App Service apps are automatically unloaded when there's no traffic. Unloading apps reduces resource
        # consumption when apps share a single App Services Plan.
        # After an app have been unloaded, the next web request will trigger a cold start of the app.
        # A cold start of the app can cause a noticeable performance issues and request timeouts.
        # Continuous WebJobs or WebJobs triggered with a CRON expression must use always on to start.
        # The Always On feature is implemented by the App Service load balancer,
        # periodically sending requests to the application root."
        name = "Ensure App Service is set to be always on"
        id = "CKV_AZURE_214"
        supported_resources = ('azurerm_linux_web_app', 'azurerm_windows_web_app')
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.PASSED)

    def get_inspected_key(self) -> str:
        return 'site_config/[0]/always_on/[0]'


check = AppServiceAlwaysOn()



from __future__ import annotations

import re
from abc import abstractmethod
from collections.abc import Iterable
from typing import Dict, Any, List, Optional

from checkov.arm.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.models.consts import ANY_VALUE
from checkov.common.util.data_structures_utils import find_in_dict

VARIABLE_DEPENDANT_REGEX = re.compile(r"(?:local|var|module)\.[^\s]+")


class BaseResourceValueCheck(BaseResourceCheck):
    def __init__(
        self,
        name: str,
        id: str,
        categories: Iterable[CheckCategories],
        supported_resources: "Iterable[str]",
        missing_block_result: CheckResult = CheckResult.FAILED,
        guideline: Optional[str] = None,
    ) -> None:
        super().__init__(
            name=name, id=id, categories=categories, supported_resources=supported_resources, guideline=guideline
        )
        self.missing_block_result = missing_block_result

    @staticmethod
    def _is_variable_dependant(value: Any) -> bool:
        if isinstance(value, str) and re.match(VARIABLE_DEPENDANT_REGEX, value):
            return True
        return False

    def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
        inspected_key = self.get_inspected_key()
        expected_values = self.get_expected_values()
        value = find_in_dict(conf, inspected_key)
        if value is not None:
            if ANY_VALUE in expected_values:
                # Key is found in the configuration - if it accepts any value, the check is PASSED
                return CheckResult.PASSED
            if isinstance(value, list) and len(value) == 1:
                value = value[0]
            if value in expected_values:
                return CheckResult.PASSED
            if self._is_variable_dependant(value):
                # If the tested attribute is variable-dependant, then result is PASSED
                return CheckResult.PASSED
            return CheckResult.FAILED
        return self.missing_block_result

    @abstractmethod
    def get_inspected_key(self) -> str:
        """
        :return: JSONPath syntax path of the checked attribute
        """
        raise NotImplementedError()

    def get_expected_values(self) -> List[Any]:
        """
        Override the method with the list of acceptable values if the check has more than one possible expected value, given
        the inspected key
        :return: List of expected values, defaults to a list of the expected value
        """
        return [self.get_expected_value()]

    def get_expected_value(self) -> Any:
        """
        Returns the default expected value, governed by provider best practices
        """
        return True



from __future__ import annotations

import fnmatch
import importlib
import importlib.util
import logging
import os
import sys
from abc import abstractmethod
from collections import defaultdict
from itertools import chain
from typing import Generator, Tuple, Dict, List, Optional, Any, TYPE_CHECKING

from checkov.common.models.enums import CheckResult
from checkov.common.resource_code_logger_filter import add_resource_code_filter_to_logger
from checkov.common.typing import _SkippedCheck, _CheckResult
from checkov.runner_filter import RunnerFilter

if TYPE_CHECKING:
    from checkov.common.checks.base_check import BaseCheck


class BaseCheckRegistry:
    # NOTE: Needs to be static to because external check loading may be triggered by a registry to which
    #       checks aren't registered. (This happens with Serverless, for example.)
    __loading_external_checks = False  # noqa: CCE003
    __all_registered_checks: list[BaseCheck] = []  # noqa: CCE003

    def __init__(self, report_type: str) -> None:
        self.logger = logging.getLogger(__name__)
        add_resource_code_filter_to_logger(self.logger)
        # IMPLEMENTATION NOTE: Checks is used to directly access checks based on an specific entity
        self.checks: Dict[str, List[BaseCheck]] = defaultdict(list)
        # IMPLEMENTATION NOTE: When using a wildcard, every pattern needs to be checked. To reduce the
        #                      number of checks checks with the same pattern are grouped, which is the
        #                      reason to use a dict for this too.
        self.wildcard_checks: Dict[str, List[BaseCheck]] = defaultdict(list)
        self.check_id_allowlist: Optional[List[str]] = None
        self.report_type = report_type
        self.definitions_raw: list[tuple[int, str]] | None = None
        self.graph = None

    def register(self, check: BaseCheck) -> None:
        # IMPLEMENTATION NOTE: Checks are registered when the script is loaded
        #                      (see BaseResourceCheck.__init__() for the various frameworks). The only
        #                      difficultly with this process is that external checks need to be specially
        #                      identified for filter handling. That's why you'll see stateful setting of
        #                      RunnerFilters during load_external_checks.
        #                      Built-in checks are registered immediately at script start, before
        #                      external checks.
        if BaseCheckRegistry.__loading_external_checks:
            RunnerFilter.notify_external_check(check.id)

        for entity in check.supported_entities:
            checks = self.wildcard_checks if self._is_wildcard(entity) else self.checks
            if not any(c.id == check.id for c in checks[entity]):
                checks[entity].append(check)

        BaseCheckRegistry.__all_registered_checks.append(check)

    @staticmethod
    def get_all_registered_checks() -> List[BaseCheck]:
        return BaseCheckRegistry.__all_registered_checks

    @staticmethod
    def _is_wildcard(entity: str) -> bool:
        return "*" in entity or "?" in entity or ("[" in entity and "]" in entity)

    def get_check_by_id(self, check_id: str) -> Optional[BaseCheck]:
        return next(
            (check for check in chain(*self.checks.values(), *self.wildcard_checks.values()) if check.id == check_id),
            None,
        )

    def all_checks(self) -> Generator[Tuple[str, BaseCheck], None, None]:
        for entity, checks in self.checks.items():
            for check in checks:
                yield entity, check
        for entity, checks in self.wildcard_checks.items():
            for check in checks:
                yield entity, check

    @property
    def contains_wildcard(self) -> bool:
        return bool(self.wildcard_checks)

    def get_checks(self, entity: str) -> List[BaseCheck]:
        if not self.wildcard_checks:
            # Optimisation: When no wildcards are used, we can use the list in self.checks
            return self.checks.get(entity) or []
        else:
            res = self.checks[entity].copy() if entity in self.checks.keys() else []
            # check wildcards
            for pattern, checks in self.wildcard_checks.items():
                if entity and fnmatch.fnmatchcase(entity, pattern):
                    res += checks
            return res

    def set_checks_allowlist(self, runner_filter: RunnerFilter) -> None:
        if runner_filter.checks:
            self.check_id_allowlist = runner_filter.checks

    @abstractmethod
    def extract_entity_details(self, entity: Dict[str, Any]) -> Tuple[str, str, Dict[str, Any]]:
        raise NotImplementedError()

    def scan(
        self,
        scanned_file: str,
        entity: Dict[str, Any],
        skipped_checks: List[_SkippedCheck],
        runner_filter: RunnerFilter,
        report_type: Optional[str] = None  # allow runners like TF plan to override the type while using the same registry
    ) -> Dict[BaseCheck, _CheckResult]:
        results: Dict[BaseCheck, _CheckResult] = {}

        try:
            (entity_type, entity_name, entity_configuration) = self.extract_entity_details(entity)
        except Exception:
            logging.debug(f"Error in entity details extraction for file {scanned_file}", exc_info=True)
            return results

        if not isinstance(entity_configuration, dict):
            return results

        checks = self.get_checks(entity_type)
        for check in checks:
            skip_info: _SkippedCheck = {}
            if skipped_checks:
                if check.id in [x["id"] for x in skipped_checks]:
                    skip_info = [x for x in skipped_checks if x["id"] == check.id][0]

            if runner_filter.should_run_check(
                    check,
                    report_type=report_type or self.report_type,
                    file_origin_paths=[scanned_file]
            ):
                result = self.run_check(check, entity_configuration, entity_name, entity_type, scanned_file, skip_info)
                results[check] = result
        return results

    def run_check(
        self,
        check: BaseCheck,
        entity_configuration: Dict[str, List[Any]],
        entity_name: str,
        entity_type: str,
        scanned_file: str,
        skip_info: _SkippedCheck,
    ) -> _CheckResult:
        self.logger.debug("Running check: {} on file {}".format(check.name, scanned_file))
        check.graph = self.graph
        try:
            result = check.run(
                scanned_file=scanned_file,
                entity_configuration=entity_configuration,
                entity_name=entity_name,
                entity_type=entity_type,
                skip_info=skip_info,
            )
            return result
        except Exception:
            return _CheckResult(
                result=CheckResult.UNKNOWN, suppress_comment="", evaluated_keys=[],
                results_configuration=entity_configuration, check=check, entity=entity_configuration
            )

    @staticmethod
    def _directory_has_init_py(directory: str) -> bool:
        """ Check if a given directory contains a file named __init__.py.

        __init__.py is needed to ensure the directory is a Python module, thus
        can be imported.
        """
        return os.path.exists(os.path.join(directory, "__init__.py"))

    @staticmethod
    def _file_can_be_imported(entry: "os.DirEntry[str]") -> bool:
        """ Verify if a directory entry is a non-magic Python file."""
        return entry.is_file() and not entry.name.startswith("__") and entry.name.endswith(".py")

    def load_external_checks(self, directory: str) -> None:
        """ Browse a directory looking for .py files to import.

        Log an error when the directory does not contains an __init__.py or
        when a .py file has syntax error
        """
        directory = os.path.expanduser(directory)
        self.logger.debug(f"Loading external checks from {directory}")
        for root, _, _ in os.walk(directory):
            sys.path.insert(1, root)
            with os.scandir(root) as directory_content:
                if not self._directory_has_init_py(root):
                    self.logger.info(f"No __init__.py found in {root}. Cannot load any check here.")
                else:
                    for entry in directory_content:
                        if self._file_can_be_imported(entry):
                            check_name = entry.name.replace(".py", "")
                            check_full_path = entry.path

                            # Filter is set while loading external checks so the filter can be informed
                            # of the checks, which need to be handled specially.
                            try:
                                BaseCheckRegistry.__loading_external_checks = True
                                self.logger.debug(f"Importing external check '{check_name}'")

                                spec = importlib.util.spec_from_file_location(check_name, check_full_path)
                                if spec:
                                    module = importlib.util.module_from_spec(spec)
                                    sys.modules[check_name] = module
                                    spec.loader.exec_module(module)  # type: ignore[union-attr] # loader can't be None here
                                else:
                                    self.logger.error(f"Cannot load external check '{check_name}' from {check_full_path}")
                            except Exception:
                                self.logger.error(f"Cannot load external check '{check_name}' from {check_full_path}", exc_info=True)
                            finally:
                                BaseCheckRegistry.__loading_external_checks = False



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class SecurityListIngressStateless(BaseResourceCheck):
    def __init__(self):
        name = "Ensure VCN inbound security lists are stateless"
        id = "CKV_OCI_17"
        supported_resources = ['oci_core_security_list']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'ingress_security_rules' in conf.keys():
            self.evaluated_keys = ['ingress_security_rules']
            rules = conf.get("ingress_security_rules")
            is_list_syntax = False
            if isinstance(rules[0], list):
                # Old terraform oci provider version syntax for ingress_security_rules was a list instead of a block
                rules = rules[0]
                is_list_syntax = True
            for idx, rule in enumerate(rules):
                if 'stateless' in rule.keys():
                    stateless = rule.get("stateless")
                    if stateless != [True] and stateless is not True:
                        self.evaluated_keys = [f'ingress_security_rules/[{idx}]/stateless'] if is_list_syntax \
                            else [f'ingress_security_rules/[0]/[{idx}]/stateless']
                        return CheckResult.FAILED
            return CheckResult.PASSED

        return CheckResult.UNKNOWN


check = SecurityListIngressStateless()



from __future__ import annotations

from abc import ABC
from typing import Dict, List, Any

from cloudsplaining.scan.policy_document import PolicyDocument

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.terraform.checks.utils.base_cloudsplaining_iam_scanner import BaseTerraformCloudsplainingIAMScanner


class BaseTerraformCloudsplainingResourceIAMCheck(BaseResourceCheck, BaseTerraformCloudsplainingIAMScanner, ABC):
    def __init__(self, name: str, id: str) -> None:
        supported_resources = (
            "aws_iam_role_policy",
            "aws_iam_user_policy",
            "aws_iam_group_policy",
            "aws_iam_policy",
            "aws_ssoadmin_permission_set_inline_policy",
        )
        super().__init__(name=name, id=id, categories=(CheckCategories.IAM,), supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        return self.scan_conf(conf)

    @property
    def cache_key(self) -> str:
        return self.entity_path

    def should_scan_conf(self, conf: Dict[str, List[Any]]) -> bool:
        if self.entity_type == "aws_ssoadmin_permission_set_inline_policy":
            return "inline_policy" in conf

        return "policy" in conf

    def convert_to_iam_policy(self, conf: Dict[str, Any]) -> PolicyDocument:
        if self.entity_type == "aws_ssoadmin_permission_set_inline_policy":
            policy: dict[str, Any] = conf['inline_policy'][0]
        else:
            policy = conf['policy'][0]

        return PolicyDocument(policy)



"""
Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""
from __future__ import annotations

import json
import logging
from pathlib import Path
from typing import Any

from checkov.common.parsers.json.decoder import Decoder
from checkov.common.parsers.json.errors import DecodeError
from checkov.common.resource_code_logger_filter import add_resource_code_filter_to_logger
from checkov.common.util.file_utils import read_file_with_any_encoding

LOGGER = logging.getLogger(__name__)
add_resource_code_filter_to_logger(LOGGER)


def load(
    filename: str | Path, allow_nulls: bool = True, content: str | None = None
) -> tuple[dict[str, Any], list[tuple[int, str]]]:
    """
    Load the given JSON file
    """

    if not content:
        content = read_file_with_any_encoding(file_path=filename)

    file_lines = [(idx + 1, line) for idx, line in enumerate(content.splitlines(keepends=True))]

    return json.loads(content, cls=Decoder, allow_nulls=allow_nulls), file_lines


def parse(
    filename: str | Path,
    allow_nulls: bool = True,
    out_parsing_errors: dict[str, str] | None = None,
    file_content: str | None = None,
) -> tuple[dict[str, Any] | list[dict[str, Any]], list[tuple[int, str]]] | None:
    error: Exception | None = None
    try:
        return load(filename=filename, allow_nulls=allow_nulls, content=file_content)
    except DecodeError as e:
        logging.debug(f"Got DecodeError parsing file {filename}", exc_info=True)
        error = e
    except json.JSONDecodeError as e:
        # Most parsing errors will get caught by the exception above. But, if the file
        # is totally empty, and perhaps in other specific cases, the json library will
        # not even begin parsing with our custom logic that throws the exception above,
        # and will fail with this exception instead.
        logging.debug(f"Got JSONDecodeError parsing file {filename}", exc_info=True)
        error = e
    except UnicodeDecodeError as e:
        logging.debug(f"Got UnicodeDecodeError parsing file {filename}", exc_info=True)
        error = e

    if error:
        if out_parsing_errors is None:
            out_parsing_errors = {}
        out_parsing_errors[str(filename)] = str(error)

    return None



from typing import Dict, Any, List

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.kubernetes.checks.resource.base_spec_check import BaseK8Check


class DropCapabilitiesPSP(BaseK8Check):
    def __init__(self) -> None:
        # CIS-1.3 1.7.7
        # CIS-1.5 5.2.7
        name = "Do not admit containers with the NET_RAW capability"
        # Location: PodSecurityPolicy.spec.requiredDropCapabilities
        id = "CKV_K8S_7"
        supported_kind = ("PodSecurityPolicy",)
        categories = (CheckCategories.KUBERNETES,)
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_kind)

    def scan_spec_conf(self, conf: Dict[str, Any]) -> CheckResult:
        spec = conf.get("spec")
        if spec and isinstance(spec, dict):
            drop_cap = spec.get("requiredDropCapabilities")
            if drop_cap and isinstance(drop_cap, list):
                if any(cap in drop_cap for cap in ("ALL", "NET_RAW")):
                    return CheckResult.PASSED

        return CheckResult.FAILED

    def get_evaluated_keys(self) -> List[str]:
        return ["spec/requiredDropCapabilities"]


check = DropCapabilitiesPSP()



from __future__ import annotations

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.util.type_forcers import force_list, extract_policy_dict
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from typing import Any


class IAMStarActionPolicyDocument(BaseResourceCheck):
    def __init__(self) -> None:
        name = 'Ensure no IAM policies documents allow "*" as a statement\'s actions'
        id = "CKV_AWS_63"
        supported_resources = (
            "aws_iam_role_policy",
            "aws_iam_user_policy",
            "aws_iam_group_policy",
            "aws_iam_policy",
            "aws_ssoadmin_permission_set_inline_policy",
        )
        categories = (CheckCategories.IAM,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        try:
            policy_block = None
            if "policy" in conf.keys():
                policy_block = extract_policy_dict(conf["policy"][0])
            elif "inline_policy" in conf.keys():
                policy_block = extract_policy_dict(conf["inline_policy"][0])
            if policy_block and "Statement" in policy_block.keys():
                for statement in force_list(policy_block["Statement"]):
                    if "Action" in statement:
                        actions = force_list(statement["Action"])
                        if (
                            isinstance(actions[0], str)
                            and statement.get("Effect", "Allow") == "Allow"
                            and "*" in actions
                        ):
                            # scanning a HCL file
                            return CheckResult.FAILED
                        elif (
                            isinstance(actions[0], list)
                            and statement.get("Effect", ["Allow"]) == ["Allow"]
                            and "*" in actions[0]
                        ):
                            # scanning a TF plan file
                            return CheckResult.FAILED
        except Exception:  # nosec
            pass
        return CheckResult.PASSED

    def get_evaluated_keys(self) -> list[str]:
        return ["policy", "inline_policy"]


check = IAMStarActionPolicyDocument()



from __future__ import annotations

from abc import abstractmethod
from typing import Any, TYPE_CHECKING

import dpath
from checkov.common.models.enums import CheckResult
from checkov.kubernetes.checks.resource.base_spec_check import BaseK8Check

if TYPE_CHECKING:
    from checkov.common.models.enums import CheckCategories
    from collections.abc import Iterable


class BaseSpecOmittedOrValueCheck(BaseK8Check):
    def __init__(
        self, name: str, id: str, categories: Iterable[CheckCategories], supported_entities: Iterable[str]
    ) -> None:
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_entities)

    def scan_spec_conf(self, conf: dict[str, Any]) -> CheckResult:
        inspected_key = self.get_inspected_key()
        if dpath.search(conf, inspected_key, yielded=False) != {}:
            if dpath.get(conf, inspected_key) != self.get_expected_value():
                return CheckResult.FAILED
        return CheckResult.PASSED

    @abstractmethod
    def get_inspected_key(self) -> str:
        raise NotImplementedError()

    def get_expected_value(self) -> Any:
        # default expected value. can be override by derived class
        return False



from __future__ import annotations

from typing import Any

from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.util.secrets import AWS, GENERAL, get_secrets_from_string


class LambdaEnvironmentCredentials(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure no hard-coded secrets exist in lambda environment"
        id = "CKV_AWS_45"
        supported_resources = ("AWS::Lambda::Function", "AWS::Serverless::Function")
        categories = (CheckCategories.SECRETS,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        self.evaluated_keys = ["Properties/Environment/Variables"]
        properties = conf.get("Properties")
        if isinstance(properties, dict):
            environment = properties.get("Environment")
            if environment and isinstance(environment, dict):
                variables = environment.get("Variables")
                if variables and isinstance(variables, dict):
                    for var_name, value in variables.items():
                        if isinstance(value, dict):
                            # if it is a resolved instrinsic function like !Ref: xyz, then it can't be a secret
                            continue

                        secrets = get_secrets_from_string(str(value), AWS, GENERAL)
                        if secrets:
                            self.evaluated_keys = [f"Properties/Environment/Variables/{var_name}"]
                            for idx, secret in enumerate(secrets):
                                conf[f'{self.id}_secret_{idx}'] = secret
                            return CheckResult.FAILED

        return CheckResult.PASSED


check = LambdaEnvironmentCredentials()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.github.base_github_configuration_check import BaseGithubCheck
from checkov.github.schemas.repository_collaborators import schema as repository_collaborators_schema
from checkov.json_doc.enums import BlockType


class GithubRepositoryCollaborators(BaseGithubCheck):
    def __init__(self) -> None:
        name = "Ensure 2 admins are set for each repository"
        id = "CKV_GITHUB_9"
        categories = [CheckCategories.SUPPLY_CHAIN]
        super().__init__(
            id=id,
            name=name,
            categories=categories,
            supported_entities=["*"],
            block_type=BlockType.DOCUMENT
        )

    def scan_entity_conf(self, conf: dict[str, Any], entity_type: str) -> CheckResult:
        ckv_metadata, conf = self.resolve_ckv_metadata_conf(conf=conf)
        if 'repository_collaborators' in ckv_metadata.get('file_name', ''):
            if conf and repository_collaborators_schema.validate(conf):
                admin_collaborators = 0
                for item in conf:
                    if isinstance(item, dict):
                        permissions = item.get("permissions", {})
                        admin = permissions.get('admin', False)
                        if admin:
                            admin_collaborators += 1
                if admin_collaborators >= 2:
                    return CheckResult.PASSED
                else:
                    return CheckResult.FAILED
        return CheckResult.UNKNOWN


check = GithubRepositoryCollaborators()



from __future__ import annotations

from typing import TYPE_CHECKING

from checkov.dockerfile.image_referencer.provider import DockerfileProvider

if TYPE_CHECKING:
    from checkov.common.images.image_referencer import Image
    from dockerfile_parse.parser import _Instruction


class DockerfileImageReferencerManager:
    __slots__ = ("definitions",)

    def __init__(self, definitions: dict[str, dict[str, list[_Instruction]]]) -> None:
        self.definitions = definitions

    def extract_images_from_resources(self) -> list[Image]:
        provider = DockerfileProvider(definitions=self.definitions)

        images = provider.extract_images_from_resources()

        return images



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck


class ElasticsearchDomainLogging(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure Elasticsearch Domain Logging is enabled"
        id = "CKV_AWS_84"
        supported_resources = ("AWS::Elasticsearch::Domain", "AWS::OpenSearchService::Domain")
        categories = (CheckCategories.LOGGING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        properties = conf.get("Properties")
        if properties:
            options = properties.get("LogPublishingOptions")
            if options:
                for option_conf in options.values():
                    if isinstance(option_conf, dict) and option_conf.get("Enabled"):
                        return CheckResult.PASSED
        return CheckResult.FAILED


check = ElasticsearchDomainLogging()



from __future__ import annotations

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class SQLDatabaseZoneRedundant(BaseResourceValueCheck):
    def __init__(self) -> None:
        """
        This is a best practise which helps to:
        - Improved High Availability: Zone redundancy ensures that your database is replicated
          across Availability Zones within an Azure region. If one Availability Zone experiences an outage,
          your database continues to operate from the other zones, minimizing downtime.
        - Reduced Maintenance Downtime: Zone-redundant configurations often require
          less planned maintenance downtime because updates and patches can be applied to
          one zone at a time while the other zones continue to serve traffic.
        - Improved Scalability: Zone-redundant configurations are designed to scale with your workload.
          You can take advantage of features like Hyperscale to dynamically adjust resources based on
          your database's performance needs.
        - Improved SLA: Azure SQL Database zone-redundant configurations typically offer
          a higher service-level agreement (SLA) for availability compared to non-zone-redundant configurations.

        However, it's critical to note that:
        Note that:
        - Zone-redundant availability is available to databases in the
          General Purpose, Premium, Business Critical and Hyperscale service tiers of the vCore purchasing model,
          and not the Basic and Standard service tiers of the DTU-based purchasing model.
        - This may not be required for:
           - Databases that supports applications which doesn't a high maturity in terms of "High Availability"
           - Databases that are very sensitive to network latency that may increase the transaction commit time,
             and thus impact the performance of some OLTP workloads.
        """
        name = "Ensure the Azure SQL Database Namespace is zone redundant"
        id = "CKV_AZURE_229"
        supported_resources = ("azurerm_mssql_database",)
        categories = (CheckCategories.BACKUP_AND_RECOVERY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "zone_redundant"


check = SQLDatabaseZoneRedundant()



from __future__ import annotations

from typing import Optional, Any, TYPE_CHECKING

if TYPE_CHECKING:
    from typing_extensions import TypeAlias

FOREACH_STRING = 'for_each'
COUNT_STRING = 'count'
REFERENCES_VALUES = r"(var|module|local)\."
FOR_EACH_BLOCK_TYPE: TypeAlias = "dict[int, Optional[list[str] | dict[str, Any] | int]]"
COUNT_KEY = 'count.index'
EACH_KEY = 'each.key'
EACH_VALUE = 'each.value'



from __future__ import annotations

from abc import abstractmethod
from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.common.util.json_utils import get_jsonpath_from_evaluated_key
from checkov.github.base_github_configuration_check import BaseGithubCheck
from checkov.github.schemas.organization import schema as org_schema
from checkov.json_doc.enums import BlockType


class BaseOrganizationCheck(BaseGithubCheck):
    def __init__(self, id: str, name: str, missing_attribute_result: CheckResult = CheckResult.PASSED) -> None:
        categories = (CheckCategories.SUPPLY_CHAIN,)
        super().__init__(
            id=id,
            name=name,
            categories=categories,
            supported_entities=("*",),
            block_type=BlockType.DOCUMENT,
        )
        self.missing_attribute_result = missing_attribute_result

    def scan_entity_conf(self, conf: dict[str, Any], entity_type: str) -> CheckResult:
        ckv_metadata, conf = self.resolve_ckv_metadata_conf(conf=conf)
        if 'org_metadata' in ckv_metadata.get('file_name', ''):
            if org_schema.validate(conf):
                evaluated_key = self.get_evaluated_keys()[0]
                jsonpath_expression = get_jsonpath_from_evaluated_key(evaluated_key)
                matches = jsonpath_expression.find(conf)
                if matches:
                    if matches[0].value in self.get_allowed_values():
                        return CheckResult.PASSED
                    return CheckResult.FAILED
                return self.missing_attribute_result
        return CheckResult.UNKNOWN

    @abstractmethod
    def get_evaluated_keys(self) -> list[str]:
        pass

    @abstractmethod
    def get_allowed_values(self) -> list[Any]:
        pass



from __future__ import annotations

from typing import Any
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.checks.enums import BlockType
from checkov.openapi.checks.base_openapi_check import BaseOpenapiCheck


class NoMaximumNumberItems(BaseOpenapiCheck):
    def __init__(self) -> None:
        id = "CKV_OPENAPI_21"
        name = "Ensure that arrays have a maximum number of items"
        categories = (CheckCategories.API_SECURITY,)
        supported_resources = ('paths',)
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_resources,
                         block_type=BlockType.DOCUMENT)

    def scan_entity_conf(self, conf: dict[str, Any], entity_type: str) -> tuple[CheckResult, dict[str, Any]]:
        result = self.check_array_max_items(inner_conf=conf)
        if result:
            return result

        return CheckResult.PASSED, conf

    def check_array_max_items(self, inner_conf: Any) -> tuple[CheckResult, dict[str, Any]] | None:
        if isinstance(inner_conf, dict):
            if "type" in inner_conf:
                if inner_conf["type"] == "array" and inner_conf.get("maxItems") is None:
                    return CheckResult.FAILED, inner_conf
            for value in inner_conf.values():
                if isinstance(value, dict):
                    result = self.check_array_max_items(inner_conf=value)
                    if result:
                        return result
                if isinstance(value, list):
                    for inner_conf_2 in value:
                        result = self.check_array_max_items(inner_conf=inner_conf_2)
                        if result:
                            return result

        return None


check = NoMaximumNumberItems()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class NeptuneDBClustersCopyTagsToSnapshots(BaseResourceValueCheck):
    def __init__(self):
        description = "Neptune DB clusters should be configured to copy tags to snapshots"
        id = "CKV_AWS_362"
        supported_resources = ['aws_neptune_cluster']
        categories = [CheckCategories.BACKUP_AND_RECOVERY]
        super().__init__(name=description, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "copy_tags_to_snapshot"


check = NeptuneDBClustersCopyTagsToSnapshots()



from __future__ import annotations

from typing import Any

from checkov.bitbucket_pipelines.base_bitbucket_pipelines_check import BaseBitbucketPipelinesCheck
from checkov.common.models.enums import CheckResult
from checkov.yaml_doc.enums import BlockType


class ImageReferenceLatestTag(BaseBitbucketPipelinesCheck):
    def __init__(self) -> None:
        name = "Ensure the pipeline image uses a non latest version tag"
        id = "CKV_BITBUCKETPIPELINES_1"
        super().__init__(
            name=name,
            id=id,
            block_type=BlockType.ARRAY,
            supported_entities=(
                "[{image:image,__startline__:__startline__,__endline__:__endline__}]",
                "pipelines.default[].step.{image: image, __startline__: __startline__, __endline__:__endline__}",
                "pipelines.*.[*][][][].step.{image: image, __startline__: __startline__, __endline__:__endline__}",
            ),
        )

    def scan_conf(self, conf: dict[str, Any]) -> tuple[CheckResult, dict[str, Any]]:
        if not isinstance(conf, dict):
            return CheckResult.UNKNOWN, conf
        image = conf.get("image")
        if not image:
            return CheckResult.UNKNOWN, conf
        if isinstance(image, str):
            if image.endswith(":latest"):
                return CheckResult.FAILED, conf

        return CheckResult.PASSED, conf


check = ImageReferenceLatestTag()



from __future__ import annotations

import logging
import os

from typing import List, Any, Optional, Dict

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.wrapper import CDK_FRAMEWORK_PREFIX
from checkov.common.checks.base_check_registry import BaseCheckRegistry
from checkov.runner_filter import RunnerFilter

logger = logging.getLogger(__name__)


CDK_CHECKS_DIR_PATH = f'{os.getcwd()}/checkov/{CDK_FRAMEWORK_PREFIX}/checks'


class Registry(BaseCheckRegistry):
    def __init__(self, checks_dir: str | None = None) -> None:
        super().__init__(report_type=CheckType.SAST)
        self.rules: List[Dict[str, Any]] = []
        self.checks_dir = checks_dir
        self.logger = logging.getLogger(__name__)
        self.runner_filter: Optional[RunnerFilter] = None
        self.checks_dirs_path: List[str] = [checks_dir] if checks_dir else []

    def extract_entity_details(self, entity: dict[str, Any]) -> tuple[str, str, dict[str, Any]]:
        # TODO
        return '', '', {}

    def set_runner_filter(self, runner_filter: RunnerFilter) -> None:
        self.runner_filter = runner_filter

    def add_external_dirs(self, external_dirs: Optional[List[str]]) -> None:
        if external_dirs:
            for path in external_dirs:
                if os.path.exists(path):
                    if not os.path.isabs(path):
                        path = os.path.abspath(path)
                    self.checks_dirs_path.append(path)
                else:
                    logger.warning(f"path: {path} not found")



from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck
from checkov.common.models.enums import CheckCategories
from typing import List, Any


class NeptuneClusterInstancePublic(BaseResourceNegativeValueCheck):
    def __init__(self):
        name = "Ensure Neptune Cluster instance is not publicly available"
        id = "CKV_AWS_102"
        supported_resources = ['aws_neptune_cluster_instance']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return 'publicly_accessible/[0]'

    def get_forbidden_values(self) -> List[Any]:
        return [True]


check = NeptuneClusterInstancePublic()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.kubernetes.checks.resource.base_spec_check import BaseK8Check


class MinimizeCapabilitiesPSP(BaseK8Check):
    def __init__(self) -> None:
        # CIS-1.5 5.2.9
        name = "Minimize the admission of containers with capabilities assigned"
        # Location: PodSecurityPolicy.spec.requiredDropCapabilities
        id = "CKV_K8S_36"
        supported_kind = ("PodSecurityPolicy",)
        categories = (CheckCategories.KUBERNETES,)
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_kind)

    def scan_spec_conf(self, conf: dict[str, Any]) -> CheckResult:
        if "spec" in conf:
            if "requiredDropCapabilities" in conf["spec"]:
                if conf["spec"]["requiredDropCapabilities"]:
                    return CheckResult.PASSED
        return CheckResult.FAILED


check = MinimizeCapabilitiesPSP()



from __future__ import annotations
from checkov.arm.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class AppServiceDetailedErrorMessagesEnabled(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that App service enables detailed error messages"
        id = "CKV_AZURE_65"
        supported_resources = ['Microsoft.Web/sites/config']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "properties/detailedErrorLoggingEnabled"


check = AppServiceDetailedErrorMessagesEnabled()



from __future__ import annotations

import itertools
import logging
from typing import List, Optional, Dict, Any, Tuple

from igraph import Graph
from rustworkx import PyDiGraph

from checkov.common.graph.checks_infra import debug

try:
    from networkx import edge_dfs, DiGraph
except ImportError:
    logging.info("Not able to import networkx")
    edge_dfs = lambda G : []

from checkov.common.graph.checks_infra.enums import Operators
from checkov.common.checks_infra.solvers.connections_solvers.base_connection_solver import BaseConnectionSolver
from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.typing import LibraryGraph, _RustworkxGraph
from checkov.terraform.graph_builder.graph_components.block_types import BlockType


class ConnectionExistsSolver(BaseConnectionSolver):
    operator = Operators.EXISTS  # noqa: CCE003  # a static attribute

    def __init__(
            self,
            resource_types: List[str],
            connected_resources_types: List[str],
            vertices_under_resource_types: Optional[List[Dict[str, Any]]] = None,
            vertices_under_connected_resources_types: Optional[List[Dict[str, Any]]] = None,
    ) -> None:
        super().__init__(
            resource_types,
            connected_resources_types,
            vertices_under_resource_types,
            vertices_under_connected_resources_types,
        )

    def get_operation(
        self, graph_connector: LibraryGraph
    ) -> Tuple[List[Dict[str, Any]], List[Dict[str, Any]], List[Dict[str, Any]]]:
        passed, failed, unknown = self._get_operation(graph_connector=graph_connector)

        debug.connection_block(
            resource_types=self.resource_types,
            connected_resource_types=self.connected_resources_types,
            operator=self.operator,
            passed_resources=passed,
            failed_resources=failed,
        )

        return passed, failed, unknown

    def _get_operation(
        self, graph_connector: LibraryGraph
    ) -> Tuple[List[Dict[str, Any]], List[Dict[str, Any]], List[Dict[str, Any]]]:
        passed: List[Dict[str, Any]] = []
        failed: List[Dict[str, Any]] = []
        unknown: List[Dict[str, Any]] = []
        if not self.vertices_under_resource_types or not self.vertices_under_connected_resources_types:
            failed.extend(self.vertices_under_resource_types)
            failed.extend(self.vertices_under_connected_resources_types)
            return passed, failed, unknown

        if isinstance(graph_connector, Graph):
            self.get_igraph_operation(graph_connector=graph_connector, passed=passed, failed=failed, unknown=unknown)
        elif isinstance(graph_connector, DiGraph):
            self.get_networkx_operation(graph_connector=graph_connector, passed=passed, failed=failed, unknown=unknown)
        elif isinstance(graph_connector, PyDiGraph):
            self.get_rustworkx_operation(graph_connector=graph_connector, passed=passed, failed=failed, unknown=unknown)
        else:
            raise Exception(f"Graph type {type(graph_connector)} not supported")

        failed.extend(
            [
                v
                for v in itertools.chain(
                    self.vertices_under_resource_types, self.vertices_under_connected_resources_types
                )
                if v not in itertools.chain(passed, unknown)
            ]
        )

        return passed, failed, unknown

    def get_igraph_operation(
        self,
        graph_connector: Graph,
        passed: list[dict[str, Any]],
        failed: list[dict[str, Any]],
        unknown: list[dict[str, Any]],
    ) -> None:
        for root_vertex in graph_connector.vs:
            inverted = False
            origin_attributes = None
            destination_attributes_list = []
            for vertex in graph_connector.dfsiter(root_vertex.index):
                resource_type = vertex[CustomAttributes.RESOURCE_TYPE]
                attributes = vertex["attr"]
                if resource_type in self.resource_types and attributes in self.vertices_under_resource_types:
                    if not origin_attributes:
                        origin_attributes = attributes
                    elif inverted:
                        destination_attributes_list.append(attributes)
                elif (
                    resource_type in self.connected_resources_types
                    and attributes in self.vertices_under_connected_resources_types
                ):
                    if not origin_attributes:
                        origin_attributes = attributes
                        inverted = True
                    else:
                        destination_attributes_list.append(attributes)

            if origin_attributes and destination_attributes_list:
                for destination_attributes in destination_attributes_list:
                    self.populate_checks_results(
                        origin_attributes=origin_attributes,
                        destination_attributes=destination_attributes,
                        passed=passed,
                        failed=failed,
                        unknown=unknown,
                    )

    def get_networkx_operation(
        self,
        graph_connector: DiGraph,
        passed: list[dict[str, Any]],
        failed: list[dict[str, Any]],
        unknown: list[dict[str, Any]],
    ) -> None:
        for u, v in edge_dfs(graph_connector):
            origin_attributes = graph_connector.nodes(data=True)[u]
            opposite_vertices = None
            if origin_attributes in self.vertices_under_resource_types:
                opposite_vertices = self.vertices_under_connected_resources_types
            elif origin_attributes in self.vertices_under_connected_resources_types:
                opposite_vertices = self.vertices_under_resource_types
            if not opposite_vertices:
                continue

            destination_attributes = graph_connector.nodes(data=True)[v]
            if destination_attributes in opposite_vertices:
                self.populate_checks_results(
                    origin_attributes=origin_attributes,
                    destination_attributes=destination_attributes,
                    passed=passed,
                    failed=failed,
                    unknown=unknown,
                )
                destination_attributes["connected_node"] = origin_attributes
                continue
            if origin_attributes.get(CustomAttributes.BLOCK_TYPE) == BlockType.OUTPUT:
                print(1)
            destination_block_type = destination_attributes.get(CustomAttributes.BLOCK_TYPE)
            if destination_block_type == BlockType.OUTPUT:
                try:
                    output_edges = graph_connector.edges(v, data=True)
                    _, output_destination, _ = next(iter(output_edges))
                    output_destination = graph_connector.nodes(data=True)[output_destination]
                    output_destination_type = output_destination.get(CustomAttributes.RESOURCE_TYPE)
                    if self.is_associated_edge(
                        origin_attributes.get(CustomAttributes.RESOURCE_TYPE), output_destination_type
                    ):
                        passed.extend([origin_attributes, output_destination])
                except StopIteration:
                    continue

    def get_rustworkx_operation(
        self,
        graph_connector: _RustworkxGraph,
        passed: list[dict[str, Any]],
        failed: list[dict[str, Any]],
        unknown: list[dict[str, Any]],
    ) -> None:
        for edge in graph_connector.edge_list():
            u, v = edge
            origin_attributes = graph_connector.nodes()[u][1]
            opposite_vertices = None
            if origin_attributes in self.vertices_under_resource_types:
                opposite_vertices = self.vertices_under_connected_resources_types
            elif origin_attributes in self.vertices_under_connected_resources_types:
                opposite_vertices = self.vertices_under_resource_types
            if not opposite_vertices:
                continue

            destination_attributes = graph_connector.nodes()[v][1]
            if destination_attributes in opposite_vertices:
                self.populate_checks_results(
                    origin_attributes=origin_attributes,
                    destination_attributes=destination_attributes,
                    passed=passed,
                    failed=failed,
                    unknown=unknown,
                )
                destination_attributes["connected_node"] = origin_attributes
                continue

            destination_block_type = destination_attributes.get(CustomAttributes.BLOCK_TYPE)
            if destination_block_type == BlockType.OUTPUT:
                try:
                    output_edges = graph_connector.adj_direction(
                        v, False
                    )  # True means inbound edges and False means outbound edges
                    output_destination_index = next(iter(output_edges))
                    output_destination = graph_connector.nodes()[output_destination_index][1]
                    output_destination_type = output_destination.get(CustomAttributes.RESOURCE_TYPE)
                    if self.is_associated_edge(
                        origin_attributes.get(CustomAttributes.RESOURCE_TYPE), output_destination_type
                    ):
                        passed.extend([origin_attributes, output_destination])
                except StopIteration:
                    continue



from __future__ import annotations

from typing import Any, Dict, TYPE_CHECKING

from checkov.kubernetes.image_referencer.provider.k8s import SUPPORTED_K8S_IMAGE_RESOURCE_TYPES
from checkov.kustomize.image_referencer.base_provider import BaseKustomizeProvider

if TYPE_CHECKING:
    from igraph import Graph
    from networkx import DiGraph


class KustomizeProvider(BaseKustomizeProvider):
    def __init__(self, graph_connector: Graph | DiGraph, report_mutator_data: Dict[str, Dict[str, Any]]):
        super().__init__(
            graph_connector=graph_connector,
            supported_resource_types=SUPPORTED_K8S_IMAGE_RESOURCE_TYPES,
            report_mutator_data=report_mutator_data
        )



from __future__ import annotations

import logging
import re
import shutil

from checkov.common.goget.base_getter import BaseGetter
from checkov.common.resource_code_logger_filter import add_resource_code_filter_to_logger
from checkov.common.util.contextmanagers import temp_environ

try:
    from git import Repo
    git_import_error = None
except ImportError as e:
    git_import_error = e

COMMIT_ID_PATTERN = re.compile(r"\?(ref=)(?P<commit_id>([0-9a-f]{40}))")
TAG_PATTERN = re.compile(r'\?(ref=)(?P<tag>(.*))')  # technically should be with ?ref=tags/ but this catches both
BRANCH_PATTERN = re.compile(r'\?(ref=heads/)(?P<branch>(.*))')


class GitGetter(BaseGetter):
    def __init__(self, url: str, create_clone_and_result_dirs: bool = True) -> None:
        self.logger = logging.getLogger(__name__)
        add_resource_code_filter_to_logger(self.logger)
        self.create_clone_and_res_dirs = create_clone_and_result_dirs
        self.tag = ''
        self.commit_id: str | None = None
        self.branch = ''

        if "?ref" in url:
            url = self.extract_git_ref(url=url)

        super().__init__(url)

    def extract_git_ref(self, url: str) -> str:
        search_branch = re.search(BRANCH_PATTERN, url)
        if search_branch:
            self.branch = search_branch.group("branch")
            # remove heads/ from ref= to get actual branch name
            # self.branch = re.sub('heads.*/', '', url)
            url = re.sub(BRANCH_PATTERN, '', url)
            return url

        search_commit_id = re.search(COMMIT_ID_PATTERN, url)
        if search_commit_id:
            self.commit_id = search_commit_id.group("commit_id")
            url = re.sub(COMMIT_ID_PATTERN, '', url)
            return url

        search_tag = re.search(TAG_PATTERN, url)
        if search_tag:
            self.tag = search_tag.group("tag")
            # remove tag/ or tags/ from ref= to get actual branch name
            self.tag = re.sub('tag.*/', '', self.tag)
            url = re.sub(TAG_PATTERN, '', url)
            return url

        logging.info(f"Module URL has an unknown ref: {url}")

        return url

    def do_get(self) -> str:
        if git_import_error is not None:
            raise ImportError("Unable to load git module (is the git executable available?)") \
                from git_import_error

        git_url, internal_dir = self._source_subdir()

        clone_dir = self.temp_dir + "/clone/" if self.create_clone_and_res_dirs else self.temp_dir
        self._clone(git_url, clone_dir)

        if internal_dir:
            clone_dir = clone_dir + internal_dir

        if self.create_clone_and_res_dirs:
            result_dir = self.temp_dir + "/result/"
            shutil.copytree(clone_dir, result_dir)
            return result_dir

        return clone_dir

    def _clone(self, git_url: str, clone_dir: str) -> None:
        self.logger.debug(f"cloning {self.url if '@' not in self.url else self.url.split('@')[1]} to {clone_dir}")
        with temp_environ(GIT_TERMINAL_PROMPT="0"):  # disables user prompts originating from GIT
            if self.branch:
                Repo.clone_from(git_url, clone_dir, branch=self.branch, depth=1)  # depth=1 for shallow clone
            elif self.commit_id:  # no commit id support for branch
                repo = Repo.clone_from(git_url, clone_dir, no_checkout=True)  # need to be a full git clone
                repo.git.checkout(self.commit_id)
            elif self.tag:
                Repo.clone_from(git_url, clone_dir, depth=1, b=self.tag)
            else:
                Repo.clone_from(git_url, clone_dir, depth=1)

    # Split source url into Git url and subdirectory path e.g. test.com/repo//repo/subpath becomes 'test.com/repo', '/repo/subpath')
    # Also see reference implementation @ go-getter https://github.com/hashicorp/go-getter/blob/main/source.go
    def _source_subdir(self) -> tuple[str, str]:
        stop = len(self.url)

        query_index = self.url.find("?")
        if query_index > -1:
            stop = query_index

        start = 0
        scheme_index = self.url.find("://", start, stop)
        if scheme_index > -1:
            start = scheme_index + 3

        subdir_index = self.url.find("//", start, stop)
        if subdir_index == -1:
            return self.url, ""

        internal_dir = self.url[subdir_index + 1:stop]  # Note: Internal dir is expected to start with /
        git_url = self.url[:subdir_index] + self.url[stop:]

        return git_url, internal_dir



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from typing import List, Any


class RDSInstanceSSL(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure RDS instance uses SSL"
        id = "CKV_ALI_20"
        supported_resources = ['alicloud_db_instance']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'ssl_action'

    def get_expected_values(self) -> List[Any]:
        return ["Open", "Update"]


check = RDSInstanceSSL()



import logging

from checkov.common.graph.graph_builder import CustomAttributes
from checkov.terraform.graph_builder.graph_components.blocks import TerraformBlock
from checkov.terraform.graph_builder.local_graph import TerraformLocalGraph
from checkov.terraform.graph_builder.graph_components.block_types import BlockType
from checkov.common.output.report import Report
from checkov.terraform.plan_parser import TF_PLAN_RESOURCE_ADDRESS
from typing import Dict, Tuple, Optional


class DeepAnalysisGraphManager:
    def __init__(self, tf_graph: TerraformLocalGraph, tf_plan_graph: TerraformLocalGraph) -> None:
        self.tf_graph: TerraformLocalGraph = tf_graph
        self.tf_plan_graph: TerraformLocalGraph = tf_plan_graph
        self._address_to_tf_idx_and_vertex_map: Dict[str, Tuple[int, TerraformBlock]] = {}
        self._address_to_tf_plan_idx_and_vertex_map: Dict[str, Tuple[int, TerraformBlock]] = {}
        self._apply_address_mapping()

    def _apply_address_mapping(self) -> None:
        self._address_to_tf_idx_and_vertex_map = {
            vertex.attributes[TF_PLAN_RESOURCE_ADDRESS]: (i, vertex)
            for i, vertex in enumerate(self.tf_graph.vertices)
            if vertex.block_type == BlockType.RESOURCE
        }
        self._address_to_tf_plan_idx_and_vertex_map = {
            vertex.attributes[TF_PLAN_RESOURCE_ADDRESS]: (i, vertex)
            for i, vertex in enumerate(self.tf_plan_graph.vertices)
            if vertex.block_type == BlockType.RESOURCE
        }

    def _get_tf_vertex_idx_from_tf_plan_vertex(self, v: TerraformBlock) -> Optional[int]:
        vertex = self._address_to_tf_idx_and_vertex_map.get(v.attributes.get(CustomAttributes.TF_RESOURCE_ADDRESS, ''))
        if vertex is None:
            return None
        return vertex[0]

    def append_vertex_to_terraform_graph(self, tf_plan_vertex: TerraformBlock, tf_plan_vertex_index: int, address: str) -> None:
        new_vertex_idx = len(self.tf_graph.vertices)
        self.tf_graph.vertices.append(tf_plan_vertex)
        self._address_to_tf_idx_and_vertex_map[address] = (new_vertex_idx, tf_plan_vertex)

        for edge in self.tf_plan_graph.out_edges[tf_plan_vertex_index]:
            dest = self.tf_plan_graph.vertices[edge.dest]
            dest_index = self._get_tf_vertex_idx_from_tf_plan_vertex(dest)
            if dest_index:
                self.tf_graph.create_edge(new_vertex_idx, dest_index, edge.label)
        for edge in self.tf_plan_graph.in_edges[tf_plan_vertex_index]:
            origin = self.tf_plan_graph.vertices[edge.origin]
            origin_index = self._get_tf_vertex_idx_from_tf_plan_vertex(origin)
            if origin_index:
                self.tf_graph.create_edge(origin_index, new_vertex_idx, edge.label)

    def enrich_tf_graph_attributes(self) -> None:
        for address, tf_plan_idx_and_vertex in self._address_to_tf_plan_idx_and_vertex_map.items():
            tf_plan_vertex_index, tf_plan_vertex = tf_plan_idx_and_vertex
            tf_idx_and_vertex = self._address_to_tf_idx_and_vertex_map.get(address)
            if not tf_idx_and_vertex:
                logging.info(f'Cant find this address: {address} in tf graph, adding it')
                self.append_vertex_to_terraform_graph(tf_plan_vertex, tf_plan_vertex_index, address)
                continue
            _, tf_vertex = tf_idx_and_vertex
            tf_vertex.attributes = {**tf_vertex.attributes, **tf_plan_vertex.attributes}
            tf_vertex.path = tf_plan_vertex.path

    def filter_report(self, report: Report) -> None:
        report.failed_checks = [check for check in report.failed_checks if
                                check.resource_address in self._address_to_tf_plan_idx_and_vertex_map]
        report.passed_checks = [check for check in report.passed_checks if
                                check.resource_address in self._address_to_tf_plan_idx_and_vertex_map]
        report.skipped_checks = [check for check in report.skipped_checks if
                                 check.resource_address in self._address_to_tf_plan_idx_and_vertex_map]
        # No need to filter other fields for now
        report.resources = set()
        report.extra_resources = set()
        report.parsing_errors = []



from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class NACLPortCheck(BaseResourceCheck):
    def __init__(self):
        name = "An inbound Network ACL rule should not allow ALL ports."
        id = "CKV_NCP_12"
        supported_resources = ('ncloud_network_acl_rule',)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'inbound' in conf.keys():
            for inbound in conf['inbound']:
                if 'port_range' in inbound.keys():
                    for port_range in inbound['port_range']:
                        if port_range == "1-65535":
                            return CheckResult.FAILED
                    return CheckResult.PASSED
        return CheckResult.FAILED


check = NACLPortCheck()



from typing import Any, Dict

from checkov.common.models.enums import CheckResult
from checkov.kubernetes.checks.resource.base_container_check import BaseK8sContainerCheck
from checkov.kubernetes.checks.resource.k8s.Tiller import Tiller


class TillerDeploymentListener(BaseK8sContainerCheck):
    def __init__(self) -> None:
        name = "Ensure the Tiller Deployment (Helm V2) is not accessible from within the cluster"
        id = "CKV_K8S_45"
        # Location: container .image
        super().__init__(name=name, id=id)

    def scan_container_conf(self, metadata: Dict[str, Any], conf: Dict[str, Any]) -> CheckResult:
        self.evaluated_container_keys = ["image"]
        if not Tiller.is_tiller(metadata, conf):
            return CheckResult.UNKNOWN

        self.evaluated_container_keys.append("arge")
        args = conf.get("args")
        if args:
            for arg in args:
                if "--listen" in arg and ("localhost" in arg or "127.0.0.1" in arg):
                    return CheckResult.PASSED

        return CheckResult.FAILED


check = TillerDeploymentListener()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.common.util.data_structures_utils import find_in_dict
from checkov.kubernetes.checks.resource.base_spec_check import BaseK8Check


class ShareHostPID(BaseK8Check):
    def __init__(self) -> None:
        # CIS-1.3 1.7.2
        # CIS-1.5 5.2.2
        name = "Containers should not share the host process ID namespace"
        id = "CKV_K8S_17"
        # Location: Pod.spec.hostPID
        # Location: CronJob.spec.jobTemplate.spec.template.spec.hostPID
        # Location: *.spec.template.spec.hostPID
        supported_kind = (
            "Pod",
            "Deployment",
            "DaemonSet",
            "StatefulSet",
            "ReplicaSet",
            "ReplicationController",
            "Job",
            "CronJob",
        )
        categories = (CheckCategories.KUBERNETES,)
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_kind)

    def scan_spec_conf(self, conf: dict[str, Any]) -> CheckResult:
        spec = {}

        if conf["kind"] == "Pod":
            if "spec" in conf:
                spec = conf["spec"]
        elif conf["kind"] == "CronJob":
            inner_spec = find_in_dict(input_dict=conf, key_path="spec/jobTemplate/spec/template/spec")
            spec = inner_spec if inner_spec else spec
        else:
            inner_spec = find_in_dict(input_dict=conf, key_path="spec/template/spec")
            spec = inner_spec if inner_spec else spec
        if spec:
            if "hostPID" in spec:
                if spec["hostPID"]:
                    return CheckResult.FAILED

        # This value is by default set to false
        return CheckResult.PASSED


check = ShareHostPID()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck
from checkov.common.models.consts import ANY_VALUE


class ComputeInstanceAdminPassword(BaseResourceNegativeValueCheck):
    def __init__(self):
        name = "Ensure that instance does not use basic credentials"
        id = "CKV_OPENSTACK_4"
        supported_resources = ['openstack_compute_instance_v2']
        categories = [CheckCategories.SECRETS]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_attribute_result=CheckResult.PASSED)

    def scan_resource_conf(self, conf):
        admin_pass = conf.get('admin_pass')
        if isinstance(admin_pass, list):
            admin_pass = admin_pass[0]
        if admin_pass:  # should be missing, or an empty string
            conf[f'{self.id}_secret'] = admin_pass
            return CheckResult.FAILED
        else:
            return CheckResult.PASSED

    def get_inspected_key(self) -> str:
        return 'admin_pass'

    def get_forbidden_values(self):
        return [ANY_VALUE]


check = ComputeInstanceAdminPassword()



from typing import Any

from checkov.common.models.consts import ANY_VALUE
from checkov.common.models.enums import CheckCategories
from checkov.arm.base_resource_value_check import BaseResourceValueCheck


class AzureManagedDiskEncryptionSet(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = (
            "Ensure that managed disks use a specific set of disk encryption sets for the "
            "customer-managed key encryption"
        )
        id = "CKV_AZURE_93"
        supported_resources = ("Microsoft.Compute/disks",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "properties/encryption/diskEncryptionSetId"

    def get_expected_value(self) -> Any:
        return ANY_VALUE


check = AzureManagedDiskEncryptionSet()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class DropCapabilitiesPSP(BaseResourceCheck):

    def __init__(self):
        # CIS-1.3 1.7.7
        # CIS-1.5 5.2.7
        name = "Do not admit containers with the NET_RAW capability"
        # Location: PodSecurityPolicy.spec.requiredDropCapabilities
        id = "CKV_K8S_7"

        supported_resources = ['kubernetes_pod_security_policy']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf) -> CheckResult:
        if conf.get('spec'):
            spec = conf.get('spec')[0]
            if not spec:
                return CheckResult.UNKNOWN

            if spec.get("required_drop_capabilities"):
                drop_cap = spec.get("required_drop_capabilities")[0]
                if drop_cap and isinstance(drop_cap, list):
                    if any(cap in drop_cap for cap in ("ALL", "NET_RAW")):
                        return CheckResult.PASSED

        return CheckResult.FAILED


check = DropCapabilitiesPSP()



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class SpannerDatabaseDropProtection(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure Spanner Database has drop protection enabled"
        id = "CKV_GCP_120"
        supported_resources = ["google_spanner_database"]
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.FAILED)

    def get_inspected_key(self) -> str:
        return "enable_drop_protection"

    def get_expected_value(self) -> bool:
        return True


check = SpannerDatabaseDropProtection()



from __future__ import annotations

from typing import Any

from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class NeptuneClusterLogging(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure Neptune logging is enabled"
        id = "CKV_AWS_101"
        supported_resources = ("AWS::Neptune::DBCluster",)
        categories = (CheckCategories.LOGGING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        logs_exports = conf.get("Properties", {}).get("EnableCloudwatchLogsExports", [])
        if "audit" in logs_exports:
            return CheckResult.PASSED

        return CheckResult.FAILED

    def get_evaluated_keys(self) -> list[str]:
        return ["Properties/EnableCloudwatchLogsExports"]


check = NeptuneClusterLogging()



from typing import Dict, Any, List

import pytest

from cdk_integration_tests.utils import run_check, load_failed_checks_from_file

LANGUAGE = 'python'


@pytest.fixture(scope="session", autouse=True)
def failed_checks() -> Dict[str, List[Dict[str, Any]]]:
    report_failed_checks = load_failed_checks_from_file(LANGUAGE)
    yield report_failed_checks


def test_CKV_AWS_18_S3BucketLogging(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_18", policy_name="S3BucketLogging", language="python")


def test_CKV_AWS_19_S3BucketEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_19", policy_name="S3BucketEncryption", language="python")


def test_CKV_AWS_21_S3BucketVersioning(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_21", policy_name="S3BucketVersioning", language="python")


def test_CKV_AWS_145_S3BucketKMSEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_145", policy_name="S3BucketKMSEncryption", language="python")


def test_CKV2_AWS_6_S3BucketPublicAccessBlock(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV2_AWS_6", policy_name="S3BucketPublicAccessBlock", language="python")


def test_CKV_AWS_54_S3BlockPublicPolicy(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_54", policy_name="S3BlockPublicPolicy", language="python")


def test_CKV_AWS_26_SNSTopicEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_26", policy_name="SNSTopicEncryption", language="python")


def test_CKV_AWS_20_S3PublicACLRead(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_20", policy_name="S3PublicACLRead", language="python")


def test_CKV_AWS_55_S3IgnorePublicACLs(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_55", policy_name="S3IgnorePublicACLs", language="python")


def test_CKV_AWS_56_S3RestrictPublicBuckets(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_56", policy_name="S3RestrictPublicBuckets", language="python")


def test_CKV_AWS_53_S3BlockPublicACLs(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_53", policy_name="S3BlockPublicACLs", language="python")


def test_CKV_AWS_57_S3PublicACLWrite(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_57", policy_name="S3PublicACLWrite", language="python")


def test_CKV_AWS_115_LambdaFunctionLevelConcurrentExecutionLimit(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_115", policy_name="LambdaFunctionLevelConcurrentExecutionLimit", language="python")


def test_CKV_AWS_116_LambdaDLQConfigured(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_116", policy_name="LambdaDLQConfigured", language="python")


def test_CKV_AWS_28_DynamodbRecovery(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_28", policy_name="DynamodbRecovery", language="python")


def test_CKV_AWS_158_CloudWatchLogGroupKMSKey(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_158", policy_name="CloudWatchLogGroupKMSKey", language="python")


def test_CKV_AWS_3_EBSEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_3", policy_name="EBSEncryption", language="python")


def test_CKV_AWS_120_APIGatewayCacheEnable(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_120", policy_name="APIGatewayCacheEnable", language="python")


def test_CKV_AWS_163_ECRImageScanning(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_163", policy_name="ECRImageScanning", language="python")


def test_CKV_AWS_51_ECRImmutableTags(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_51", policy_name="ECRImmutableTags", language="python")


def test_CKV_AWS_44_NeptuneClusterStorageEncrypted(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_44", policy_name="NeptuneClusterStorageEncrypted", language="python")


def test_CKV_AWS_166_BackupVaultEncrypted(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_166", policy_name="BackupVaultEncrypted", language="python")


def test_CKV_AWS_74_DocDBEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_74", policy_name="DocDBEncryption", language="python")


def test_CKV_AWS_47_DAXEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_47", policy_name="DAXEncryption", language="python")


def test_CKV_AWS_156_WorkspaceRootVolumeEncrypted(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_156", policy_name="WorkspaceRootVolumeEncrypted", language="python")


def test_CKV_AWS_155_WorkspaceUserVolumeEncrypted(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_155", policy_name="WorkspaceUserVolumeEncrypted", language="python")


def test_CKV_AWS_165_DynamodbGlobalTableRecovery(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_165", policy_name="DynamodbGlobalTableRecovery", language="python")


def test_CKV_AWS_27_SQSQueueEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_27", policy_name="SQSQueueEncryption", language="python")


def test_CKV_AWS_195_GlueSecurityConfigurationEnabled(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_195", policy_name="GlueSecurityConfigurationEnabled", language="python")


def test_CKV_AWS_30_ElasticacheReplicationGroupEncryptionAtTransit(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_30", policy_name="ElasticacheReplicationGroupEncryptionAtTransit", language="python")


def test_CKV_AWS_29_ElasticacheReplicationGroupEncryptionAtRest(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_29", policy_name="ElasticacheReplicationGroupEncryptionAtRest", language="python")


def test_CKV_AWS_43_KinesisStreamEncryptionType(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_43", policy_name="KinesisStreamEncryptionType", language="python")


def test_CKV_AWS_42_EFSEncryptionEnabled(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_42", policy_name="EFSEncryptionEnabled", language="python")


def test_CKV_AWS_193_AppSyncLogging(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_193", policy_name="AppSyncLogging", language="python")


def test_CKV_AWS_194_AppSyncFieldLevelLogs(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_194", policy_name="AppSyncFieldLevelLogs", language="python")


def test_CKV_AWS_104_DocDBAuditLogs(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_104", policy_name="DocDBAuditLogs", language="python")


def test_CKV_AWS_82_AthenaWorkgroupConfiguration(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_82", policy_name="AthenaWorkgroupConfiguration", language="python")


def test_CKV_AWS_17_RDSPubliclyAccessible(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_17", policy_name="RDSPubliclyAccessible", language="python")


def test_CKV_AWS_87_RedshiftClusterPubliclyAccessible(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_87", policy_name="RedshiftClusterPubliclyAccessible", language="python")


def test_CKV_AWS_69_AmazonMQBrokerPublicAccess(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_69", policy_name="AmazonMQBrokerPublicAccess", language="python")


def test_CKV_AWS_118_RDSEnhancedMonitorEnabled(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_118", policy_name="RDSEnhancedMonitorEnabled", language="python")


def test_CKV_AWS_40_IAMPolicyAttachedToGroupOrRoles(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_40", policy_name="IAMPolicyAttachedToGroupOrRoles", language="python")


def test_CKV_AWS_36_CloudTrailLogValidation(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_36", policy_name="CloudTrailLogValidation", language="python")


def test_CKV_AWS_83_ElasticsearchDomainEnforceHTTPS(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_83", policy_name="ElasticsearchDomainEnforceHTTPS", language="python")


def test_CKV_AWS_76_APIGatewayAccessLogging(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_76", policy_name="APIGatewayAccessLogging", language="python")


def test_CKV_AWS_117_LambdaInVPC(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_117", policy_name="LambdaInVPC", language="python")


def test_CKV_AWS_68_WAFEnabled(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_68", policy_name="WAFEnabled", language="python")


def test_CKV_AWS_64_RedshiftClusterEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_64", policy_name="RedshiftClusterEncryption", language="python")


def test_CKV_AWS_78_CodeBuildProjectEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_78", policy_name="CodeBuildProjectEncryption", language="python")


def test_CKV_AWS_31_ElasticacheReplicationGroupEncryptionAtTransitAuthToken(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_31", policy_name="ElasticacheReplicationGroupEncryptionAtTransitAuthToken", language="python")


def test_CKV_AWS_94_GlueDataCatalogEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_94", policy_name="GlueDataCatalogEncryption", language="python")


def test_CKV_AWS_99_GlueSecurityConfiguration(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_99", policy_name="GlueSecurityConfiguration", language="python")


def test_CKV_AWS_105_RedShiftSSL(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_105", policy_name="RedShiftSSL", language="python")


def test_CKV_AWS_149_SecretManagerSecretEncrypted(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_149", policy_name="SecretManagerSecretEncrypted", language="python")


def test_CKV_AWS_59_APIGatewayAuthorization(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_59", policy_name="APIGatewayAuthorization", language="python")


def test_CKV_AWS_89_DMSReplicationInstancePubliclyAccessible(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_89", policy_name="DMSReplicationInstancePubliclyAccessible", language="python")


def test_CKV_AWS_34_CloudfrontDistributionEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_34", policy_name="CloudfrontDistributionEncryption", language="python")


def test_CKV_AWS_95_APIGatewayV2AccessLogging(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_95", policy_name="APIGatewayV2AccessLogging", language="python")


def test_CKV_AWS_86_CloudfrontDistributionLogging(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_86", policy_name="CloudfrontDistributionLogging", language="python")


def test_CKV_AWS_90_DocDBTLS(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_90", policy_name="DocDBTLS", language="python")


def test_CKV_AWS_174_CloudFrontTLS12(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_174", policy_name="CloudFrontTLS12", language="python")


def test_CKV_AWS_71_RedshiftClusterLogging(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_71", policy_name="RedshiftClusterLogging", language="python")


def test_CKV_AWS_92_ELBAccessLogs(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_92", policy_name="ELBAccessLogs", language="python")


def test_CKV_AWS_67_CloudtrailMultiRegion(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_67", policy_name="CloudtrailMultiRegion", language="python")


def test_CKV_AWS_91_ELBv2AccessLogs(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_91", policy_name="ELBv2AccessLogs", language="python")


def test_CKV_AWS_164_TransferServerIsPublic(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_164", policy_name="TransferServerIsPublic", language="python")


def test_CKV_AWS_97_ECSTaskDefinitionEFSVolumeEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_97", policy_name="ECSTaskDefinitionEFSVolumeEncryption", language="python")


def test_CKV_AWS_123_VPCEndpointAcceptanceConfigured(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_123", policy_name="VPCEndpointAcceptanceConfigured", language="python")


def test_CKV_AWS_35_CloudtrailEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_35", policy_name="CloudtrailEncryption", language="python")


def test_CKV_AWS_154_RedshiftInEc2ClassicMode(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_154", policy_name="RedshiftInEc2ClassicMode", language="python")


def test_CKV_AWS_84_ElasticsearchDomainLogging(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_84", policy_name="ElasticsearchDomainLogging", language="python")


def test_CKV_AWS_136_ECRRepositoryEncrypted(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_136", policy_name="ECRRepositoryEncrypted", language="python")


def test_CKV_AWS_66_CloudWatchLogGroupRetention(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_66", policy_name="CloudWatchLogGroupRetention", language="python")


def test_CKV_AWS_5_ElasticsearchEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_5", policy_name="ElasticsearchEncryption", language="python")


def test_CKV_AWS_73_APIGatewayXray(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_73", policy_name="APIGatewayXray", language="python")


def test_CKV_AWS_6_ElasticsearchNodeToNodeEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_6", policy_name="ElasticsearchNodeToNodeEncryption", language="python")


def test_CKV_AWS_88_EC2PublicIP(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_88", policy_name="EC2PublicIP", language="python")


def test_CKV_AWS_8_LaunchConfigurationEBSEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_8", policy_name="LaunchConfigurationEBSEncryption", language="python")


def test_CKV_AWS_45_LambdaEnvironmentCredentials(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_45", policy_name="LambdaEnvironmentCredentials", language="python")


def test_CKV_AWS_58_EKSSecretsEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_58", policy_name="EKSSecretsEncryption", language="python")


def test_CKV_AWS_65_ECSClusterContainerInsights(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_65", policy_name="ECSClusterContainerInsights", language="python")


def test_CKV_AWS_131_ALBDropHttpHeaders(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_131", policy_name="ALBDropHttpHeaders", language="python")


def test_CKV_AWS_2_ALBListenerHTTPS(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_2", policy_name="ALBListenerHTTPS", language="python")


def test_CKV_AWS_23_SecurityGroupRuleDescription(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_23", policy_name="SecurityGroupRuleDescription", language="python")


def test_CKV_AWS_173_LambdaEnvironmentEncryptionSettings(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_173", policy_name="LambdaEnvironmentEncryptionSettings", language="python")


def test_CKV_AWS_157_RDSMultiAZEnabled(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_157", policy_name="RDSMultiAZEnabled", language="python")


def test_CKV_AWS_96_AuroraEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_96", policy_name="AuroraEncryption", language="python")



from __future__ import annotations

from typing import Any

from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class GKEPodSecurityPolicyEnabled(BaseResourceCheck):

    """
     Pod Security Policy was removed from GKE clusters with version >= 1.25.0
    """

    def __init__(self):
        name = "Ensure PodSecurityPolicy controller is enabled on the Kubernetes Engine Clusters"
        id = "CKV_GCP_24"
        supported_resources = ['google_container_cluster']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:

        if conf.get('min_master_version') and isinstance(conf.get('min_master_version'), list):
            raw = conf.get('min_master_version')[0]
            splitter = raw.split(".")
            if len(splitter) >= 2:
                str_version = splitter[0] + "." + splitter[1]
                version = float(str_version)
                if version < 1.25:
                    if conf.get('pod_security_policy_config') and isinstance(conf.get('pod_security_policy_config'), list):
                        policy = conf.get('pod_security_policy_config')[0]
                        if policy.get('enabled') and isinstance(policy.get('enabled'), list):
                            secure = policy.get('enabled')[0]
                            if secure:
                                return CheckResult.PASSED
                    return CheckResult.FAILED
        return CheckResult.UNKNOWN


check = GKEPodSecurityPolicyEnabled()



from typing import Optional, Any, Dict

from checkov.common.graph.checks_infra.enums import Operators
from checkov.common.checks_infra.solvers.attribute_solvers.base_attribute_solver import BaseAttributeSolver


class AnyResourceSolver(BaseAttributeSolver):
    operator = Operators.ANY  # noqa: CCE003  # a static attribute
    is_value_attribute_check = False  # noqa: CCE003  # a static attribute

    def _get_operation(self, vertex: Dict[str, Any], attribute: Optional[str]) -> bool:
        return vertex is not None



from __future__ import annotations
from typing import Any
from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck

import itertools


class AzureContainerInstanceEnvVarSecureValueType(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure that Azure container environment variables are configured with secure values only"
        id = "CKV_AZURE_235"
        supported_resources = ("azurerm_container_group",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:

        for container in itertools.chain(conf.get('container', {}), conf.get('init_container', {})):
            if "environment_variables" in container:
                return CheckResult.FAILED
            return CheckResult.PASSED


check = AzureContainerInstanceEnvVarSecureValueType()



from __future__ import annotations

from typing import Dict, Set, Any

from checkov.common.sca.reachability.package_alias_mapping.abstract_alias_mapping_strategy import AbstractAliasMappingStrategy
from checkov.common.sca.reachability.package_alias_mapping.nodejs.nodejs_alias_mapping_strategy import NodejsAliasMappingStrategy

language_to_strategy: Dict[str, AbstractAliasMappingStrategy] = {
    "nodejs": NodejsAliasMappingStrategy()
}


class AliasMappingCreator:
    def __init__(self) -> None:
        self._alias_mapping: Dict[str, Any] = {"languages": {}}

    def update_alias_mapping_for_repository(
            self,
            repository_name: str,
            repository_root_dir: str,
            relevant_packages: Set[str]
    ) -> None:
        for lang in language_to_strategy:
            language_to_strategy[lang].update_alias_mapping(self._alias_mapping, repository_name, repository_root_dir, relevant_packages)

    def get_alias_mapping(self) -> Dict[str, Any]:
        return self._alias_mapping



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class KeyVaultEnablesPurgeProtection(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure that key vault enables purge protection"
        id = "CKV_AZURE_110"
        supported_resources = ['azurerm_key_vault']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "purge_protection_enabled"


check = KeyVaultEnablesPurgeProtection()



from typing import Optional, Dict, Set, List, Tuple, Any, Union

from checkov.common.bridgecrew.severities import Severity
from checkov.common.output.record import Record
from checkov.common.typing import _CheckResult


class GithubActionsRecord(Record):
    def __init__(self,
                 triggers: Optional[Set[str]],
                 job: Union[Optional[str], None],
                 workflow_name: Optional[str],
                 check_id: str,
                 check_name: str,
                 check_result: _CheckResult,
                 code_block: List[Tuple[int, str]],
                 file_path: str,
                 file_line_range: List[int],
                 resource: str,
                 evaluations: Optional[Dict[str, Any]],
                 check_class: str,
                 file_abs_path: str,
                 entity_tags: Optional[Dict[str, str]] = None,
                 bc_check_id: Optional[str] = None,
                 severity: Optional[Severity] = None,
                 details: Optional[List[str]] = None
                 ) -> None:
        super().__init__(check_id=check_id,
                         check_name=check_name,
                         check_result=check_result,
                         code_block=code_block,
                         file_path=file_path,
                         file_line_range=file_line_range,
                         resource=resource,
                         evaluations=evaluations,
                         check_class=check_class,
                         file_abs_path=file_abs_path,
                         entity_tags=entity_tags,
                         bc_check_id=bc_check_id,
                         severity=severity,
                         details=details
                         )
        self.triggers = triggers,
        self.job = job,
        self.workflow_name = workflow_name



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class PubsubSKUSLA(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        name = "Ensure Web PubSub uses a SKU with an SLA"
        id = "CKV_AZURE_175"
        supported_resources = ("azurerm_web_pubsub",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "sku"

    def get_forbidden_values(self) -> list[Any]:
        return ["Free_F1"]


check = PubsubSKUSLA()



from __future__ import annotations

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class EventHubNamespaceZoneRedundant(BaseResourceValueCheck):
    def __init__(self) -> None:
        """
        This is a best practice as the all-active Azure Event Hubs cluster model with availability zone support provides
        resiliency against grave hardware failures and even catastrophic loss of entire datacenter facilities.
        If an Event Hubs namespace is created in a region with availability zones,
        the outage risk is further spread across three physically separated facilities, and the service has enough
        capacity reserves to instantly cope up with the complete, catastrophic loss of the entire facility.

        When a client application sends events to an Event Hubs without specifying a partition, events are automatically
        distributed among partitions in the event hub. If a partition isn't available for some reason, events are
        distributed among the remaining partitions. This behavior allows for the greatest amount of up time.
        """
        name = "Ensure the Azure Event Hub Namespace is zone redundant"
        id = "CKV_AZURE_228"
        supported_resources = ("azurerm_eventhub_namespace",)
        categories = (CheckCategories.BACKUP_AND_RECOVERY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "zone_redundant"


check = EventHubNamespaceZoneRedundant()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AppServiceSlotHTTPSOnly(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure web app redirects all HTTP traffic to HTTPS in Azure App Service Slot"
        id = "CKV_AZURE_153"
        supported_resources = ["azurerm_app_service_slot", "azurerm_linux_web_app_slot", "azurerm_windows_web_app_slot"]
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "https_only/[0]"


check = AppServiceSlotHTTPSOnly()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AKSEncryptionAtHostEnabled(BaseResourceValueCheck):
    def __init__(self) -> None:
        """
        With host-based encryption, the data stored on the VM host of
        your AKS agent nodes' VMs is encrypted at rest and flows encrypted to the Storage service.

        This means the temp disks are encrypted at rest with platform-managed keys.
        The cache of OS and data disks is encrypted at rest with either platform-managed keys
        or customer-managed keys depending on the encryption type set on those disks.
        """
        name = "Ensure that the AKS cluster encrypt temp disks, caches, and data flows "
        name += "between Compute and Storage resources"
        id = "CKV_AZURE_227"
        supported_resources = ("azurerm_kubernetes_cluster", "azurerm_kubernetes_cluster_node_pool")
        categories = (CheckCategories.KUBERNETES,)
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
            missing_block_result=CheckResult.FAILED,
        )

    def get_inspected_key(self) -> str:
        if self.entity_type == "azurerm_kubernetes_cluster":
            return "default_node_pool/[0]/enable_host_encryption"
        else:
            return "enable_host_encryption"


check = AKSEncryptionAtHostEnabled()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck
from typing import List, Any


class DatabricksWorkspaceIsNotPublic(BaseResourceNegativeValueCheck):
    def __init__(self):
        name = "Ensure that databricks workspace is not public"
        id = "CKV_AZURE_158"
        supported_resources = ['azurerm_databricks_workspace']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_attribute_result=CheckResult.FAILED)

    def get_inspected_key(self) -> str:
        return 'public_network_access_enabled'

    def get_forbidden_values(self) -> List[Any]:
        return [True]


check = DatabricksWorkspaceIsNotPublic()
